/**
 * Logger for list.
 * @memberof module:apeman-wch/lib/logging
 * @constructor ListLogger
 * @returns A logger instance.
 */

"use strict";

var colorprint = require('colorprint'),
    colors = colorprint.colors,
    util = require('util'),
    path = require('path'),
    arrayfilter = require('arrayfilter'),
    arraysort = require('arraysort'),
    textconv = require('textconv');

var NAME_MIN_LENGTH = 24;
var DESC_MAX_LENGTH = 64;

/** @lends ListLogger */
module.exports = colorprint.define(
    /** @lends ListLogger.prototype */
    {
        /**
         * Log watch list.
         * @param {object} wchs - Watchs to show.
         * @param {object} childWchs - Child watches to show.
         */
        logList: function (wchs, childWchs) {
            var s = this;
            s.debug('');
            s.debug('  Available wchs:');
            s.debug('');
            Object.keys(wchs).forEach(function (name) {
                var desc = s._descWch(wchs[name]);
                s._logWch(name, desc);
            });
            s.debug('');
            (childWchs || []).forEach(function (data) {
                var hasWatches = Object.keys(data.wchs || {}).length > 0;
                if (!hasWatches) {
                    return;
                }
                var namespace = data.namespace;
                var cwd = data.cwd;
                cwd = (cwd && path.relative(process.cwd(), cwd)) || null;
                var location = (cwd ? '"./' + cwd + '"' : '');
                s.debug(colors.black.italic('    /* Watches in ' + (location || namespace) + ' */'));
                Object.keys(data.wchs)
                    .sort(arraysort.stringCompare())
                    .filter(function (name) {
                        var wchs = data.wchs[name];
                        return Object.keys(wchs || {}).length > 0;
                    })
                    .forEach(function (name) {
                        var wchs = data.wchs[name];
                        var fullName = [namespace, name].join('#');
                        var desc = s._descWch(wchs);
                        s._logWch(fullName, desc);
                    });
                s.debug('');
            });
            s.debug('');
        },
        _logWch: function (name, desc) {
            var s = this;
            desc = textconv.inline(desc, {joiner: " ", trim: true});
            desc = textconv.chop(desc, DESC_MAX_LENGTH, {ellipsis: true});
            var comment = desc ? colors.blackBright(['//', desc].join(' ')) : '';
            s.debug("    %s%s", colors.black.bold(textconv.pad(name, NAME_MIN_LENGTH)), comment);
        },
        _descWch: function (watch) {
            var s = this;
            if (Array.isArray(watch)) {
                return watch.map(function (name) {
                    return s._descWch(name) || ("`" + String(name) + "`");
                }).join(', ');
            }
            var $desc = watch.$desc || watch.$description || [
                    watch.pattern,
                    watch.action
                ].filter(arrayfilter.emptyReject()).join(' => ');
            if ($desc) {
                return $desc;
            }
            if (typeof(watch) === 'function') {
                return util.format('function(){/*...*/}');
            }
            return watch.desc || '';
        }
    }
);
